@@ -34,15 +34,13 @@ import pandas
 from pandas.api.types import is_list_like, is_bool
 from pandas.core.dtypes.common import is_integer
 from pandas.core.indexing import IndexingError
+from modin.error_message import ErrorMessage
 
 from .dataframe import DataFrame
 from .series import Series
 from .utils import is_scalar
 
 
-# TODO: rename "scaler" to scalar everywhere in this file
-
-
 def is_slice(x):
     """
     Check that argument is an instance of slice.
@@ -188,52 +186,6 @@ This will be fixed in future releases.
 """
 
 
-def _parse_tuple(tup):
-    """
-    Unpack the user input for getitem and setitem and compute ndim.
-
-    loc[a] -> ([a], :), 1D
-    loc[[a,b],] -> ([a,b], :),
-    loc[a,b] -> ([a], [b]), 0D
-
-    Parameters
-    ----------
-    tup : tuple
-        User input to unpack.
-
-    Returns
-    -------
-    row_loc : list
-        List of row locators.
-    col_list : list
-        List of column locators.
-    ndim : {0, 1, 2}
-        Number of dimensions of located dataset.
-    row_scaler : bool
-        True if `row_loc` is a scalar, False otherwise.
-    col_scaler : bool
-        True if `col_loc` is a scalar, False otherwise.
-    """
-    row_loc, col_loc = slice(None), slice(None)
-
-    if is_tuple(tup):
-        row_loc = tup[0]
-        if len(tup) == 2:
-            col_loc = tup[1]
-        if len(tup) > 2:
-            raise IndexingError("Too many indexers")
-    else:
-        row_loc = tup
-
-    ndim = _compute_ndim(row_loc, col_loc)
-    row_scaler = is_scalar(row_loc)
-    col_scaler = is_scalar(col_loc)
-    row_loc = [row_loc] if row_scaler else row_loc
-    col_loc = [col_loc] if col_scaler else col_loc
-
-    return row_loc, col_loc, ndim, row_scaler, col_scaler
-
-
 def _compute_ndim(row_loc, col_loc):
     """
     Compute the number of dimensions of result from locators.
@@ -250,12 +202,12 @@ def _compute_ndim(row_loc, col_loc):
     {0, 1, 2}
         Number of dimensions in located dataset.
     """
-    row_scaler = is_scalar(row_loc) or is_tuple(row_loc)
-    col_scaler = is_scalar(col_loc) or is_tuple(col_loc)
+    row_scalar = is_scalar(row_loc) or is_tuple(row_loc)
+    col_scalar = is_scalar(col_loc) or is_tuple(col_loc)
 
-    if row_scaler and col_scaler:
+    if row_scalar and col_scalar:
         ndim = 0
-    elif row_scaler ^ col_scaler:
+    elif row_scalar ^ col_scalar:
         ndim = 1
     else:
         ndim = 2
@@ -276,8 +228,8 @@ class _LocationIndexerBase(object):
     def __init__(self, modin_df):
         self.df = modin_df
         self.qc = modin_df._query_compiler
-        self.row_scaler = False
-        self.col_scaler = False
+        self.row_scalar = False
+        self.col_scalar = False
 
     def __getitem__(self, row_lookup, col_lookup, ndim):
         """
@@ -285,9 +237,9 @@ class _LocationIndexerBase(object):
 
         Parameters
         ----------
-        row_lookup : slice or scalar
+        row_lookup : slice(None), range or np.ndarray
             The global row index to retrieve data from.
-        col_lookup : slice or scalar
+        col_lookup : slice(None), range or np.ndarray
             The global col index to retrieve data from.
         ndim : {0, 1, 2}
             Number of dimensions in dataset to be retrieved.
@@ -296,11 +248,30 @@ class _LocationIndexerBase(object):
         -------
         modin.pandas.DataFrame or modin.pandas.Series
             Located dataset.
+
+        Notes
+        -----
+        Usage of `slice(None)` as a lookup is a hack to pass information about
+        full-axis grab without computing actual indices that triggers lazy computations.
+        Ideally, this API should get rid of using slices as indexers and either use a
+        common ``Indexer`` object or range and ``np.ndarray`` only.
         """
+        if isinstance(row_lookup, slice):
+            ErrorMessage.catch_bugs_and_request_email(
+                failure_condition=row_lookup != slice(None),
+                extra_log=f"Only None-slices are acceptable as a slice argument in masking, got: {row_lookup}",
+            )
+            row_lookup = None
+        if isinstance(col_lookup, slice):
+            ErrorMessage.catch_bugs_and_request_email(
+                failure_condition=col_lookup != slice(None),
+                extra_log=f"Only None-slices are acceptable as a slice argument in masking, got: {col_lookup}",
+            )
+            col_lookup = None
         qc_view = self.qc.view(row_lookup, col_lookup)
         if ndim == 2:
             return self.df.__constructor__(query_compiler=qc_view)
-        if isinstance(self.df, Series) and not self.row_scaler:
+        if isinstance(self.df, Series) and not self.row_scalar:
             return self.df.__constructor__(query_compiler=qc_view)
         if isinstance(self.df, Series):
             axis = 0
@@ -309,9 +280,9 @@ class _LocationIndexerBase(object):
         else:
             axis = (
                 None
-                if self.col_scaler and self.row_scaler
+                if self.col_scalar and self.row_scalar
                 else 1
-                if self.col_scaler
+                if self.col_scalar
                 else 0
             )
         return self.df.__constructor__(query_compiler=qc_view).squeeze(axis=axis)
@@ -362,7 +333,31 @@ class _LocationIndexerBase(object):
             else:
                 new_col_len = len(col_lookup)
             to_shape = new_row_len, new_col_len
-            item = self._broadcast_item(row_lookup, col_lookup, item, to_shape)
+            # If we are asssigning to a single categorical column, we won't be
+            # able to set the column to a 2-d array due to a bug in Pandas.
+            # Until that bug is fixed, don't convert `item` to a 2-d array
+            # in that case.
+            # TODO(https://github.com/pandas-dev/pandas/issues/44703): Delete
+            # this special case once the pandas bug is fixed.
+            assigning_to_single_category_column = new_col_len == 1 and (
+                # Case 1: df = pd.DataFrame({"status": ["a", "b", "c"]},
+                #                           dtype="category")
+                # Then type(df.dtypes) is pandas.core.series.Series
+                (
+                    isinstance(self.df.dtypes, pandas.core.series.Series)
+                    and self.df.dtypes[col_lookup][0].name == "category"
+                )
+                # Case 2: df = pd.Series( ["a", "b", "c"],  dtype="category")
+                # Then df.dtypes ==  CategoricalDtype(categories=['a', 'b', 'c'],
+                #                                     ordered=False)
+                # and type(df.dtypes) is pandas.core.dtypes.dtypes.CategoricalDtype.
+                # Case 3:  df = pd.Series([0, 1, 2], index=['a', 'b', 'c'])
+                # Then df.dtypes == dtype('int64') and type(df.dtypes) is
+                # numpy.dtype
+                or (getattr(self.df.dtypes, "name", "") == "category")
+            )
+            if not assigning_to_single_category_column:
+                item = self._broadcast_item(row_lookup, col_lookup, item, to_shape)
             self._write_items(row_lookup, col_lookup, item)
 
     def _broadcast_item(self, row_lookup, col_lookup, item, to_shape):
@@ -446,7 +441,7 @@ class _LocationIndexerBase(object):
         new_qc = self.qc.write_items(row_lookup, col_lookup, item)
         self.df._create_or_update_from_compiler(new_qc, inplace=True)
 
-    def _determine_setitem_axis(self, row_lookup, col_lookup, row_scaler, col_scaler):
+    def _determine_setitem_axis(self, row_lookup, col_lookup, row_scalar, col_scalar):
         """
         Determine an axis along which we should do an assignment.
 
@@ -456,9 +451,9 @@ class _LocationIndexerBase(object):
             Indexer for rows.
         col_lookup : slice or list
             Indexer for columns.
-        row_scaler : bool
+        row_scalar : bool
             Whether indexer for rows is scalar or not.
-        col_scaler : bool
+        col_scalar : bool
             Whether indexer for columns is scalar or not.
 
         Returns
@@ -473,7 +468,7 @@ class _LocationIndexerBase(object):
         axis = None: assignment along both axes
         """
         if self.df.shape == (1, 1):
-            return None if not (row_scaler ^ col_scaler) else 1 if row_scaler else 0
+            return None if not (row_scalar ^ col_scalar) else 1 if row_scalar else 0
 
         def get_axis(axis):
             return self.qc.index if axis == 0 else self.qc.columns
@@ -497,6 +492,43 @@ class _LocationIndexerBase(object):
             axis = None
         return axis
 
+    def _parse_row_and_column_locators(self, tup):
+        """
+        Unpack the user input for getitem and setitem and compute ndim.
+
+        loc[a] -> ([a], :), 1D
+        loc[[a,b]] -> ([a,b], :),
+        loc[a,b] -> ([a], [b]), 0D
+
+        Parameters
+        ----------
+        tup : tuple
+            User input to unpack.
+
+        Returns
+        -------
+        row_loc : scalar or list
+            Row locator(s) as a scalar or List.
+        col_list : scalar or list
+            Column locator(s) as a scalar or List.
+        ndim : {0, 1, 2}
+            Number of dimensions of located dataset.
+        """
+        row_loc, col_loc = slice(None), slice(None)
+
+        if is_tuple(tup):
+            row_loc = tup[0]
+            if len(tup) == 2:
+                col_loc = tup[1]
+            if len(tup) > 2:
+                raise IndexingError("Too many indexers")
+        else:
+            row_loc = tup
+
+        row_loc = row_loc(self.df) if callable(row_loc) else row_loc
+        col_loc = col_loc(self.df) if callable(col_loc) else col_loc
+        return row_loc, col_loc, _compute_ndim(row_loc, col_loc)
+
 
 class _LocIndexer(_LocationIndexerBase):
     """
@@ -514,7 +546,7 @@ class _LocIndexer(_LocationIndexerBase):
 
         Parameters
         ----------
-        key : callable or tuple
+        key : callable, scalar, or tuple
             The global row index to retrieve data from.
 
         Returns
@@ -526,9 +558,11 @@ class _LocIndexer(_LocationIndexerBase):
         --------
         pandas.DataFrame.loc
         """
-        if callable(key):
-            return self.__getitem__(key(self.df))
-        row_loc, col_loc, ndim, self.row_scaler, self.col_scaler = _parse_tuple(key)
+        if self.df.empty:
+            return self.df._default_to_pandas(lambda df: df.loc[key])
+        row_loc, col_loc, ndim = self._parse_row_and_column_locators(key)
+        self.row_scalar = is_scalar(row_loc)
+        self.col_scalar = is_scalar(col_loc)
         if isinstance(row_loc, slice) and row_loc == slice(None):
             # If we're only slicing columns, handle the case with `__getitem__`
             if not isinstance(col_loc, slice):
@@ -551,29 +585,35 @@ class _LocIndexer(_LocationIndexerBase):
         if isinstance(result, Series):
             result._parent = self.df
             result._parent_axis = 0
+        col_loc_as_list = [col_loc] if self.col_scalar else col_loc
+        row_loc_as_list = [row_loc] if self.row_scalar else row_loc
         # Pandas drops the levels that are in the `loc`, so we have to as well.
         if hasattr(result, "index") and isinstance(result.index, pandas.MultiIndex):
             if (
                 isinstance(result, Series)
-                and not isinstance(col_loc, slice)
+                and not isinstance(col_loc_as_list, slice)
                 and all(
-                    col_loc[i] in result.index.levels[i] for i in range(len(col_loc))
+                    col_loc_as_list[i] in result.index.levels[i]
+                    for i in range(len(col_loc_as_list))
                 )
             ):
-                result.index = result.index.droplevel(list(range(len(col_loc))))
-            elif not isinstance(row_loc, slice) and all(
-                not isinstance(row_loc[i], slice)
-                and row_loc[i] in result.index.levels[i]
-                for i in range(len(row_loc))
+                result.index = result.index.droplevel(list(range(len(col_loc_as_list))))
+            elif not isinstance(row_loc_as_list, slice) and all(
+                not isinstance(row_loc_as_list[i], slice)
+                and row_loc_as_list[i] in result.index.levels[i]
+                for i in range(len(row_loc_as_list))
             ):
-                result.index = result.index.droplevel(list(range(len(row_loc))))
+                result.index = result.index.droplevel(list(range(len(row_loc_as_list))))
         if (
             hasattr(result, "columns")
-            and not isinstance(col_loc, slice)
+            and not isinstance(col_loc_as_list, slice)
             and isinstance(result.columns, pandas.MultiIndex)
-            and all(col_loc[i] in result.columns.levels[i] for i in range(len(col_loc)))
+            and all(
+                col_loc_as_list[i] in result.columns.levels[i]
+                for i in range(len(col_loc_as_list))
+            )
         ):
-            result.columns = result.columns.droplevel(list(range(len(col_loc))))
+            result.columns = result.columns.droplevel(list(range(len(col_loc_as_list))))
         # This is done for cases where the index passed in has other state, like a
         # frequency in the case of DateTimeIndex.
         if (
@@ -600,7 +640,7 @@ class _LocIndexer(_LocationIndexerBase):
         --------
         pandas.DataFrame.loc
         """
-        row_loc, col_loc, _, row_scaler, col_scaler = _parse_tuple(key)
+        row_loc, col_loc, _ = self._parse_row_and_column_locators(key)
         if isinstance(row_loc, list) and len(row_loc) == 1:
             if row_loc[0] not in self.qc.index:
                 index = self.qc.index.insert(len(self.qc.index), row_loc[0])
@@ -623,7 +663,7 @@ class _LocIndexer(_LocationIndexerBase):
                 col_lookup,
                 item,
                 axis=self._determine_setitem_axis(
-                    row_lookup, col_lookup, row_scaler, col_scaler
+                    row_lookup, col_lookup, is_scalar(row_loc), is_scalar(col_loc)
                 ),
             )
 
@@ -666,9 +706,9 @@ class _LocIndexer(_LocationIndexerBase):
 
         Parameters
         ----------
-        row_loc : slice, list, array or tuple
+        row_loc : scalar, slice, list, array or tuple
             Row locator.
-        col_loc : slice, list, array or tuple
+        col_loc : scalar, slice, list, array or tuple
             Columns locator.
 
         Returns
@@ -678,6 +718,8 @@ class _LocIndexer(_LocationIndexerBase):
         col_lookup : numpy.ndarray
             List of columns labels.
         """
+        row_loc = [row_loc] if is_scalar(row_loc) else row_loc
+        col_loc = [col_loc] if is_scalar(col_loc) else col_loc
         if is_list_like(row_loc) and len(row_loc) == 1:
             if (
                 isinstance(self.qc.index.values[0], np.datetime64)
@@ -744,9 +786,11 @@ class _iLocIndexer(_LocationIndexerBase):
         --------
         pandas.DataFrame.iloc
         """
-        if callable(key):
-            return self.__getitem__(key(self.df))
-        row_loc, col_loc, ndim, self.row_scaler, self.col_scaler = _parse_tuple(key)
+        if self.df.empty:
+            return self.df._default_to_pandas(lambda df: df.iloc[key])
+        row_loc, col_loc, ndim = self._parse_row_and_column_locators(key)
+        self.row_scalar = is_scalar(row_loc)
+        self.col_scalar = is_scalar(col_loc)
         self._check_dtypes(row_loc)
         self._check_dtypes(col_loc)
 
@@ -772,7 +816,9 @@ class _iLocIndexer(_LocationIndexerBase):
         --------
         pandas.DataFrame.iloc
         """
-        row_loc, col_loc, _, row_scaler, col_scaler = _parse_tuple(key)
+        row_loc, col_loc, _ = self._parse_row_and_column_locators(key)
+        row_scalar = is_scalar(row_loc)
+        col_scalar = is_scalar(col_loc)
         self._check_dtypes(row_loc)
         self._check_dtypes(col_loc)
 
@@ -782,7 +828,7 @@ class _iLocIndexer(_LocationIndexerBase):
             col_lookup,
             item,
             axis=self._determine_setitem_axis(
-                row_lookup, col_lookup, row_scaler, col_scaler
+                row_lookup, col_lookup, row_scalar, col_scalar
             ),
         )
 
@@ -799,32 +845,39 @@ class _iLocIndexer(_LocationIndexerBase):
 
         Returns
         -------
-        row_lookup : numpy.ndarray
+        row_lookup : slice(None) if full axis grab, pandas.RangeIndex if repetition is detected, numpy.ndarray otherwise
             List of index labels.
-        col_lookup : numpy.ndarray
+        col_lookup : slice(None) if full axis grab, pandas.RangeIndex if repetition is detected, numpy.ndarray otherwise
             List of columns labels.
+
+        Notes
+        -----
+        Usage of `slice(None)` as a resulting lookup is a hack to pass information about
+        full-axis grab without computing actual indices that triggers lazy computations.
+        Ideally, this API should get rid of using slices as indexers and either use a
+        common ``Indexer`` object or range and ``np.ndarray`` only.
         """
-        if (
-            not isinstance(row_loc, slice)
-            or isinstance(row_loc, slice)
-            and row_loc.step is not None
-        ):
-            row_lookup = (
-                pandas.RangeIndex(len(self.qc.index)).to_series().iloc[row_loc].index
-            )
-        else:
-            row_lookup = row_loc
-        if (
-            not isinstance(col_loc, slice)
-            or isinstance(col_loc, slice)
-            and col_loc.step is not None
-        ):
-            col_lookup = (
-                pandas.RangeIndex(len(self.qc.columns)).to_series().iloc[col_loc].index
-            )
-        else:
-            col_lookup = col_loc
-        return row_lookup, col_lookup
+        row_loc = [row_loc] if is_scalar(row_loc) else row_loc
+        col_loc = [col_loc] if is_scalar(col_loc) else col_loc
+        lookups = []
+        for axis, axis_loc in enumerate((row_loc, col_loc)):
+            if isinstance(axis_loc, slice) and axis_loc.step is None:
+                axis_lookup = (
+                    axis_loc
+                    if axis_loc == slice(None)
+                    else pandas.RangeIndex(
+                        *axis_loc.indices(len(self.qc.get_axis(axis)))
+                    )
+                )
+            else:
+                axis_lookup = (
+                    pandas.RangeIndex(len(self.qc.get_axis(axis)))
+                    .to_series()
+                    .iloc[axis_loc]
+                    .index
+                )
+            lookups.append(axis_lookup)
+        return lookups
 
     def _check_dtypes(self, locator):
         """
