@@ -374,11 +374,7 @@ def test_fillna_dataframe():
 
     # df2 may have different index and columns
     df2 = pandas.DataFrame(
-        {
-            "a": [np.nan, 10, 20, 30, 40],
-            "b": [50, 60, 70, 80, 90],
-            "foo": ["bar"] * 5,
-        },
+        {"a": [np.nan, 10, 20, 30, 40], "b": [50, 60, 70, 80, 90], "foo": ["bar"] * 5},
         index=list("VWXuZ"),
     )
     modin_df2 = pd.DataFrame(df2)
