@@ -12,10 +12,13 @@
 # governing permissions and limitations under the License.
 
 import pytest
+import itertools
 import pandas
 import numpy as np
+import itertools
 import modin.pandas as pd
-from modin.utils import try_cast_to_pandas, get_current_backend, hashable
+from modin.utils import try_cast_to_pandas, get_current_execution, hashable
+from modin.core.dataframe.algebra.default2pandas.groupby import GroupBy
 from modin.pandas.utils import from_pandas, is_scalar
 from .utils import (
     df_equals,
@@ -26,6 +29,8 @@ from .utils import (
     test_data_values,
     modin_df_almost_equals_pandas,
     generate_multiindex,
+    test_groupby_data,
+    dict_equals,
 )
 from modin.config import NPartitions
 
@@ -33,7 +38,14 @@ NPartitions.put(4)
 
 
 def modin_groupby_equals_pandas(modin_groupby, pandas_groupby):
-    for g1, g2 in zip(modin_groupby, pandas_groupby):
+    eval_general(
+        modin_groupby, pandas_groupby, lambda grp: grp.indices, comparator=dict_equals
+    )
+    eval_general(
+        modin_groupby, pandas_groupby, lambda grp: grp.groups, comparator=dict_equals
+    )
+
+    for g1, g2 in itertools.zip_longest(modin_groupby, pandas_groupby):
         assert g1[0] == g2[0]
         df_equals(g1[1], g2[1])
 
@@ -84,7 +96,7 @@ def test_mixed_dtypes_groupby(as_index):
     ]
 
     for by in by_values:
-        if by_values[0] == "col3":
+        if isinstance(by[0], str) and by[0] == "col3":
             modin_groupby = modin_df.set_index(by[0]).groupby(
                 by=by[0], as_index=as_index
             )
@@ -127,8 +139,6 @@ def test_mixed_dtypes_groupby(as_index):
 
         # TODO Add more apply functions
         apply_functions = [lambda df: df.sum(), min]
-        # Workaround for Pandas bug #34656. Recreate groupby object for Pandas
-        pandas_groupby = pandas_df.groupby(by=by[-1], as_index=as_index)
         for func in apply_functions:
             eval_apply(modin_groupby, pandas_groupby, func)
 
@@ -163,6 +173,10 @@ def test_mixed_dtypes_groupby(as_index):
             {"col2": sum},
             {"col2": "max", "col4": "sum", "col5": "min"},
             {"col2": max, "col4": sum, "col5": "min"},
+            # Intersection of 'by' and agg cols for MapReduce impl
+            {"col0": "count", "col1": "count", "col2": "count"},
+            # Intersection of 'by' and agg cols for FullAxis impl
+            {"col0": "nunique", "col1": "nunique", "col2": "nunique"},
         ]
         for func in agg_functions:
             eval_agg(modin_groupby, pandas_groupby, func)
@@ -363,10 +377,34 @@ def test_simple_row_groupby(by, as_index, col1_category):
         eval_var(modin_groupby, pandas_groupby)
         eval_skew(modin_groupby, pandas_groupby)
 
-    agg_functions = [lambda df: df.sum(), "min", "max", min, sum]
+    agg_functions = [
+        lambda df: df.sum(),
+        "min",
+        "max",
+        min,
+        sum,
+        # Intersection of 'by' and agg cols for MapReduce impl
+        {"col1": "count", "col2": "count"},
+        # Intersection of 'by' and agg cols for FullAxis impl
+        {"col1": "nunique", "col2": "nunique"},
+    ]
     for func in agg_functions:
-        eval_agg(modin_groupby, pandas_groupby, func)
-        eval_aggregate(modin_groupby, pandas_groupby, func)
+        # Pandas raises an exception when 'by' contains categorical key and `as_index=False`
+        # because of this bug: https://github.com/pandas-dev/pandas/issues/36698
+        # Modin correctly processes the result, that's why `check_exception_type=None` in some cases
+        is_pandas_bug_case = not as_index and col1_category and isinstance(func, dict)
+        eval_general(
+            modin_groupby,
+            pandas_groupby,
+            lambda grp: grp.agg(func),
+            check_exception_type=None if is_pandas_bug_case else True,
+        )
+        eval_general(
+            modin_groupby,
+            pandas_groupby,
+            lambda grp: grp.aggregate(func),
+            check_exception_type=None if is_pandas_bug_case else True,
+        )
 
     eval_general(modin_groupby, pandas_groupby, lambda df: df.last(), is_default=True)
     eval_general(
@@ -424,7 +462,7 @@ def test_simple_row_groupby(by, as_index, col1_category):
     )
     eval_fillna(modin_groupby, pandas_groupby)
     eval_count(modin_groupby, pandas_groupby)
-    if get_current_backend() != "BaseOnPython":
+    if get_current_execution() != "BaseOnPython":
         eval_general(
             modin_groupby,
             pandas_groupby,
@@ -1216,11 +1254,25 @@ def eval_shift(modin_groupby, pandas_groupby):
         pandas_groupby,
         lambda groupby: groupby.shift(periods=-3),
     )
-    eval_general(
-        modin_groupby,
-        pandas_groupby,
-        lambda groupby: groupby.shift(axis=1, fill_value=777),
-    )
+
+    if isinstance(pandas_groupby, pandas.core.groupby.DataFrameGroupBy):
+        pandas_res = pandas_groupby.shift(axis=1, fill_value=777)
+        modin_res = modin_groupby.shift(axis=1, fill_value=777)
+        # Pandas produces unexpected index order (pandas GH 44269).
+        # Here we align index of Modin result with pandas to make test passed.
+        import pandas.core.algorithms as algorithms
+
+        indexer, _ = modin_res.index.get_indexer_non_unique(modin_res.index._values)
+        indexer = algorithms.unique1d(indexer)
+        modin_res = modin_res.take(indexer)
+
+        df_equals(modin_res, pandas_res)
+    else:
+        eval_general(
+            modin_groupby,
+            pandas_groupby,
+            lambda groupby: groupby.shift(axis=1, fill_value=777),
+        )
 
 
 def test_groupby_on_index_values_with_loop():
@@ -1281,8 +1333,9 @@ def test_groupby_multiindex(groupby_kwargs):
         modin_df = modin_df.T
         pandas_df = pandas_df.T
 
-    md_grp, pd_grp = modin_df.groupby(**groupby_kwargs), pandas_df.groupby(
-        **groupby_kwargs
+    md_grp, pd_grp = (
+        modin_df.groupby(**groupby_kwargs),
+        pandas_df.groupby(**groupby_kwargs),
     )
     modin_groupby_equals_pandas(md_grp, pd_grp)
     df_equals(md_grp.sum(), pd_grp.sum())
@@ -1366,7 +1419,7 @@ def test_shift_freq(groupby_axis, shift_axis):
                     "min": (list(test_data["int_data"].keys())[-1], min),
                 },
             },
-            marks=pytest.mark.skip("See Modin issue #2542"),
+            marks=pytest.mark.skip("See Modin issue #3602"),
         ),
     ],
 )
@@ -1389,7 +1442,7 @@ def test_agg_func_None_rename(by_and_agg_dict, as_index):
         True,
         pytest.param(
             False,
-            marks=pytest.mark.xfail_backends(
+            marks=pytest.mark.xfail_executions(
                 ["BaseOnPython"], reason="See Pandas issue #39103"
             ),
         ),
@@ -1401,14 +1454,24 @@ def test_agg_func_None_rename(by_and_agg_dict, as_index):
     [["sum", "min", "max"], ["mean", "quantile"]],
     ids=["reduction", "aggregation"],
 )
-def test_dict_agg_rename_mi_columns(as_index, by_length, agg_fns):
+@pytest.mark.parametrize(
+    "intersection_with_by_cols",
+    [pytest.param(True, marks=pytest.mark.skip("See Modin issue #3602")), False],
+)
+def test_dict_agg_rename_mi_columns(
+    as_index, by_length, agg_fns, intersection_with_by_cols
+):
     md_df, pd_df = create_test_dfs(test_data["int_data"])
     mi_columns = generate_multiindex(len(md_df.columns), nlevels=4)
 
     md_df.columns, pd_df.columns = mi_columns, mi_columns
 
     by = list(md_df.columns[:by_length])
-    agg_cols = list(md_df.columns[by_length : by_length + 3])
+    agg_cols = (
+        list(md_df.columns[by_length - 1 : by_length + 2])
+        if intersection_with_by_cols
+        else list(md_df.columns[by_length : by_length + 3])
+    )
 
     agg_dict = {
         f"custom-{i}" + str(agg_fns[i % len(agg_fns)]): (col, agg_fns[i % len(agg_fns)])
@@ -1655,7 +1718,20 @@ def test_unknown_groupby(columns):
                     list(test_data_values[0].keys())[-1]: (sum, min, max),
                 }
             ),
-            marks=pytest.mark.skip("See modin issue #2542"),
+            id="Agg_and_by_intersection_MapReduce_implementation",
+        ),
+        pytest.param(
+            lambda grp: grp.agg(
+                {
+                    list(test_data_values[0].keys())[1]: (max, "mean", "nunique"),
+                    list(test_data_values[0].keys())[-1]: (sum, min, max),
+                }
+            ),
+            id="Agg_and_by_intersection_FullAxis_implementation",
+        ),
+        pytest.param(
+            lambda grp: grp.agg({list(test_data_values[0].keys())[0]: "count"}),
+            id="Agg_and_by_intersection_issue_3376",
         ),
     ],
 )
@@ -1677,8 +1753,9 @@ def test_multi_column_groupby_different_partitions(
         md_df = md_df.astype({by[0]: "category"})
         pd_df = pd_df.astype({by[0]: "category"})
 
-    md_grp, pd_grp = md_df.groupby(by, as_index=as_index), pd_df.groupby(
-        by, as_index=as_index
+    md_grp, pd_grp = (
+        md_df.groupby(by, as_index=as_index),
+        pd_df.groupby(by, as_index=as_index),
     )
     eval_general(md_grp, pd_grp, func_to_apply)
     eval___getitem__(md_grp, pd_grp, md_df.columns[1])
@@ -1704,8 +1781,9 @@ def test_not_str_by(by, as_index):
     columns = pandas.Index([0, 1.5, "str", pandas.Timestamp("2020-02-02"), None])
 
     md_df, pd_df = create_test_dfs(data, columns=columns)
-    md_grp, pd_grp = md_df.groupby(by, as_index=as_index), pd_df.groupby(
-        by, as_index=as_index
+    md_grp, pd_grp = (
+        md_df.groupby(by, as_index=as_index),
+        pd_df.groupby(by, as_index=as_index),
     )
 
     modin_groupby_equals_pandas(md_grp, pd_grp)
@@ -1714,3 +1792,202 @@ def test_not_str_by(by, as_index):
     eval_general(md_grp, pd_grp, lambda grp: grp.agg(lambda df: df.mean()))
     eval_general(md_grp, pd_grp, lambda grp: grp.dtypes)
     eval_general(md_grp, pd_grp, lambda grp: grp.first())
+
+
+@pytest.mark.parametrize("internal_by_length", [0, 1, 2])
+@pytest.mark.parametrize("external_by_length", [0, 1, 2])
+@pytest.mark.parametrize("has_categorical_by", [True, False])
+@pytest.mark.parametrize(
+    "agg_func",
+    [
+        pytest.param(
+            lambda grp: grp.apply(lambda df: df.dtypes), id="modin_dtypes_impl"
+        ),
+        pytest.param(lambda grp: grp.apply(lambda df: df.sum()), id="apply_sum"),
+        pytest.param(lambda grp: grp.count(), id="count"),
+        pytest.param(lambda grp: grp.nunique(), id="nunique"),
+        # Integer key means the index of the column to replace it with.
+        # 0 and -1 are considered to be the indices of the columns to group on.
+        pytest.param({1: "sum", 2: "nunique"}, id="dict_agg_no_intersection_with_by"),
+        pytest.param(
+            {0: "mean", 1: "sum", 2: "nunique"},
+            id="dict_agg_has_intersection_with_by",
+        ),
+        pytest.param(
+            {1: "sum", 2: "nunique", -1: "nunique"},
+            id="dict_agg_has_intersection_with_categorical_by",
+        ),
+    ],
+)
+# There are two versions of the `handle_as_index` method: the one accepting pandas.DataFrame from
+# the execution kernel and backend agnostic. This parameter indicates which one implementation to use.
+@pytest.mark.parametrize("use_backend_agnostic_method", [True, False])
+def test_handle_as_index(
+    internal_by_length,
+    external_by_length,
+    has_categorical_by,
+    agg_func,
+    use_backend_agnostic_method,
+    request,
+):
+    """
+    Test ``modin.core.dataframe.algebra.default2pandas.groupby.GroupBy.handle_as_index``.
+
+    The role of the ``handle_as_index`` method is to build a groupby result considering
+    ``as_index=False`` from the result that was computed with ``as_index=True``.
+
+    So the testing flow is the following:
+        1. Compute GroupBy result with the ``as_index=True`` parameter via Modin.
+        2. Build ``as_index=False`` result from the ``as_index=True`` using ``handle_as_index`` method.
+        3. Compute GroupBy result with the ``as_index=False`` parameter via pandas as the reference result.
+        4. Compare the result from the second step with the reference.
+    """
+    by_length = internal_by_length + external_by_length
+    if by_length == 0:
+        pytest.skip("No keys to group on were passed, skipping the test.")
+
+    if (
+        has_categorical_by
+        and by_length > 1
+        and (
+            isinstance(agg_func, dict)
+            or ("nunique" in request.node.callspec.id.split("-"))
+        )
+    ):
+        pytest.skip(
+            "The linked bug makes pandas raise an exception when 'by' is categorical: "
+            "https://github.com/pandas-dev/pandas/issues/36698"
+        )
+
+    df = pandas.DataFrame(test_groupby_data)
+    external_by_cols = GroupBy.validate_by(df.add_prefix("external_"))
+
+    if has_categorical_by:
+        df = df.astype({df.columns[-1]: "category"})
+
+    if isinstance(agg_func, dict):
+        agg_func = {df.columns[key]: value for key, value in agg_func.items()}
+        selection = list(agg_func.keys())
+        agg_dict = agg_func
+        agg_func = lambda grp: grp.agg(agg_dict)  # noqa: E731 (lambda assignment)
+    else:
+        selection = None
+
+    # Selecting 'by' columns from both sides of the frame so they located in different partitions
+    internal_by = df.columns[
+        range(-internal_by_length // 2, internal_by_length // 2)
+    ].tolist()
+    external_by = external_by_cols[:external_by_length]
+
+    pd_by = internal_by + external_by
+    md_by = internal_by + [pd.Series(ser) for ser in external_by]
+
+    grp_result = pd.DataFrame(df).groupby(md_by, as_index=True)
+    grp_reference = df.groupby(pd_by, as_index=False)
+
+    agg_result = agg_func(grp_result)
+    agg_reference = agg_func(grp_reference)
+
+    if use_backend_agnostic_method:
+        reset_index, drop, lvls_to_drop, cols_to_drop = GroupBy.handle_as_index(
+            result_cols=agg_result.columns,
+            result_index_names=agg_result.index.names,
+            internal_by_cols=internal_by,
+            by_cols_dtypes=df[internal_by].dtypes.values,
+            by_length=len(md_by),
+            selection=selection,
+            drop=len(internal_by) != 0,
+        )
+
+        if len(lvls_to_drop) > 0:
+            agg_result.index = agg_result.index.droplevel(lvls_to_drop)
+        if len(cols_to_drop) > 0:
+            agg_result = agg_result.drop(columns=cols_to_drop)
+        if reset_index:
+            agg_result = agg_result.reset_index(drop=drop)
+    else:
+        GroupBy.handle_as_index_for_dataframe(
+            result=agg_result,
+            internal_by_cols=internal_by,
+            by_cols_dtypes=df[internal_by].dtypes.values,
+            by_length=len(md_by),
+            selection=selection,
+            drop=len(internal_by) != 0,
+            inplace=True,
+        )
+
+    df_equals(agg_result, agg_reference)
+
+
+def test_validate_by():
+    """Test ``modin.core.dataframe.algebra.default2pandas.groupby.GroupBy.validate_by``."""
+
+    def compare(obj1, obj2):
+        assert type(obj1) == type(
+            obj2
+        ), f"Both objects must be instances of the same type: {type(obj1)} != {type(obj2)}."
+        if isinstance(obj1, list):
+            for val1, val2 in itertools.zip_longest(obj1, obj2):
+                df_equals(val1, val2)
+        else:
+            df_equals(obj1, obj2)
+
+    # This emulates situation when the Series's query compiler being passed as a 'by':
+    #   1. The Series at the QC level is represented as a single-column frame with the "__reduced__" columns.
+    #   2. The valid representation of such QC is an unnamed Series.
+    reduced_frame = pandas.DataFrame({"__reduced__": [1, 2, 3]})
+    series_result = GroupBy.validate_by(reduced_frame)
+    series_reference = [pandas.Series([1, 2, 3], name=None)]
+    compare(series_reference, series_result)
+
+    # This emulates situation when several 'by' columns of the group frame are passed as a single QueryCompiler:
+    #   1. If grouping on several columns the 'by' at the QC level is the following: ``df[by]._query_compiler``.
+    #   2. The valid representation of such QC is a list of Series.
+    splited_df = [pandas.Series([1, 2, 3], name=f"col{i}") for i in range(3)]
+    splited_df_result = GroupBy.validate_by(
+        pandas.concat(splited_df, axis=1, copy=True)
+    )
+    compare(splited_df, splited_df_result)
+
+    # This emulates situation of mixed by (two column names and an external Series):
+    by = ["col1", "col2", pandas.DataFrame({"__reduced__": [1, 2, 3]})]
+    result_by = GroupBy.validate_by(by)
+    reference_by = ["col1", "col2", pandas.Series([1, 2, 3], name=None)]
+    compare(reference_by, result_by)
+
+
+@pytest.mark.parametrize("sort", [True, False])
+@pytest.mark.parametrize("is_categorical_by", [True, False])
+def test_groupby_sort(sort, is_categorical_by):
+    # from issue #3571
+    by = np.array(["a"] * 50000 + ["b"] * 10000 + ["c"] * 1000)
+    random_state = np.random.RandomState(seed=42)
+    random_state.shuffle(by)
+
+    data = {"key_col": by, "data_col": np.arange(len(by))}
+    md_df, pd_df = create_test_dfs(data)
+
+    if is_categorical_by:
+        md_df = md_df.astype({"key_col": "category"})
+        pd_df = pd_df.astype({"key_col": "category"})
+
+    md_grp = md_df.groupby("key_col", sort=sort)
+    pd_grp = pd_df.groupby("key_col", sort=sort)
+
+    modin_groupby_equals_pandas(md_grp, pd_grp)
+    eval_general(md_grp, pd_grp, lambda grp: grp.sum())
+    eval_general(md_grp, pd_grp, lambda grp: grp.size())
+    eval_general(md_grp, pd_grp, lambda grp: grp.agg(lambda df: df.mean()))
+    eval_general(md_grp, pd_grp, lambda grp: grp.dtypes)
+    eval_general(md_grp, pd_grp, lambda grp: grp.first())
+
+
+def test_sum_with_level():
+    data = {
+        "A": ["0.0", "1.0", "2.0", "3.0", "4.0"],
+        "B": ["0.0", "1.0", "0.0", "1.0", "0.0"],
+        "C": ["foo1", "foo2", "foo3", "foo4", "foo5"],
+        "D": pandas.bdate_range("1/1/2009", periods=5),
+    }
+    modin_df, pandas_df = pd.DataFrame(data), pandas.DataFrame(data)
+    eval_general(modin_df, pandas_df, lambda df: df.set_index("C").groupby("C").sum())
