@@ -15,14 +15,16 @@ import pytest
 import numpy as np
 import math
 import pandas
+import itertools
 from pandas.testing import (
     assert_series_equal,
     assert_frame_equal,
     assert_index_equal,
     assert_extension_array_equal,
 )
+from pandas.core.dtypes.common import is_list_like
 import modin.pandas as pd
-from modin.utils import to_pandas
+from modin.utils import to_pandas, try_cast_to_pandas
 from modin.config import TestDatasetSize, TrackFileLeaks
 from io import BytesIO
 import os
@@ -45,6 +47,7 @@ DATASET_SIZE_DICT = {
 
 # Size of test dataframes
 NCOLS, NROWS = DATASET_SIZE_DICT.get(TestDatasetSize.get(), DATASET_SIZE_DICT["Normal"])
+NGROUPS = 10
 
 # Range for values for test data
 RAND_LOW = 0
@@ -136,6 +139,8 @@ test_bool_data = {
     for i in range(NCOLS)
 }
 
+test_groupby_data = {f"col{i}": np.arange(NCOLS) % NGROUPS for i in range(NROWS)}
+
 test_data_resample = {
     "data": {"A": range(12), "B": range(12)},
     "index": pandas.date_range("31/12/2000", periods=12, freq="H"),
@@ -466,6 +471,27 @@ def df_categories_equals(df1, df2):
         )
 
 
+def assert_empty_frame_equal(df1, df2):
+    """
+    Test if df1 and df2 are empty.
+
+    Parameters
+    ----------
+    df1 : pandas.DataFrame or pandas.Series
+    df2 : pandas.DataFrame or pandas.Series
+
+    Raises
+    ------
+    AssertionError
+        If check fails.
+    """
+
+    if (df1.empty and not df2.empty) or (df2.empty and not df1.empty):
+        assert False, "One of the passed frames is empty, when other isn't"
+    elif df1.empty and df2.empty and type(df1) != type(df2):
+        assert False, f"Empty frames have different types: {type(df1)} != {type(df2)}"
+
+
 def df_equals(df1, df2):
     """Tests if df1 and df2 are equal.
 
@@ -516,12 +542,7 @@ def df_equals(df1, df2):
         df2 = to_pandas(df2)
 
     if isinstance(df1, pandas.DataFrame) and isinstance(df2, pandas.DataFrame):
-        if (df1.empty and not df2.empty) or (df2.empty and not df1.empty):
-            assert False, "One of the passed frames is empty, when other isn't"
-        elif df1.empty and df2.empty and type(df1) != type(df2):
-            assert (
-                False
-            ), f"Empty frames have different types: {type(df1)} != {type(df2)}"
+        assert_empty_frame_equal(df1, df2)
 
     if isinstance(df1, pandas.DataFrame) and isinstance(df2, pandas.DataFrame):
         assert_frame_equal(
@@ -757,6 +778,7 @@ def eval_io(
             pd,
             pandas,
             applyier,
+            comparator=comparator,
             check_exception_type=check_exception_type,
             raising_exceptions=raising_exceptions,
             check_kwargs_callable=check_kwargs_callable,
@@ -1107,11 +1129,7 @@ def generate_dataframe(row_size=NROWS, additional_col_values=None):
 
     if additional_col_values is not None:
         assert isinstance(additional_col_values, (list, tuple))
-        data.update(
-            {
-                "col7": random_state.choice(additional_col_values, size=row_size),
-            }
-        )
+        data.update({"col7": random_state.choice(additional_col_values, size=row_size)})
     return pandas.DataFrame(data)
 
 
@@ -1228,23 +1246,33 @@ def teardown_test_files(test_paths: list):
         teardown_test_file(path)
 
 
-def sort_index_for_equal_values(series, ascending=False):
-    if series.index.dtype == np.float64:
+def sort_index_for_equal_values(df, ascending=True):
+    """Sort `df` indices of equal rows."""
+    if df.index.dtype == np.float64:
         # HACK: workaround for pandas bug:
         # https://github.com/pandas-dev/pandas/issues/34455
-        series.index = series.index.astype("str")
-    res = series.groupby(series, sort=False).apply(
+        df.index = df.index.astype("str")
+    res = df.groupby(by=df if df.ndim == 1 else df.columns, sort=False).apply(
         lambda df: df.sort_index(ascending=ascending)
     )
-    if res.index.nlevels > series.index.nlevels:
+    if res.index.nlevels > df.index.nlevels:
         # Sometimes GroupBy adds an extra level with 'by' to the result index.
         # GroupBy is very inconsistent about when it's doing this, so that's
         # why this clumsy if-statement is used.
         res.index = res.index.droplevel(0)
-    res.name = series.name
+    # GroupBy overwrites original index names with 'by', so the following line restores original names
+    res.index.names = df.index.names
     return res
 
 
+def df_equals_with_non_stable_indices(df1, df2):
+    """Assert equality of two frames regardless of the index order for equal values."""
+    df1, df2 = map(try_cast_to_pandas, (df1, df2))
+    np.testing.assert_array_equal(df1.values, df2.values)
+    sorted1, sorted2 = map(sort_index_for_equal_values, (df1, df2))
+    df_equals(sorted1, sorted2)
+
+
 def rotate_decimal_digits_or_symbols(value):
     if value.dtype == object:
         # When dtype is object, we assume that it is actually strings from MultiIndex level names
@@ -1253,3 +1281,73 @@ def rotate_decimal_digits_or_symbols(value):
         tens = value // 10
         ones = value % 10
         return tens + ones * 10
+
+
+def make_default_file(file_type: str):
+    """Helper function for pytest fixtures."""
+    filenames = []
+
+    def _create_file(filenames, filename, force, nrows, ncols, func: str, func_kw=None):
+        """
+        Helper function that creates a dataframe before writing it to a file.
+
+        Eliminates the duplicate code that is needed before of output functions calls.
+
+        Notes
+        -----
+        Importantly, names of created files are added to `filenames` variable for
+        their further automatic deletion. Without this step, files created by
+        `pytest` fixtures will not be deleted.
+        """
+        if force or not os.path.exists(filename):
+            df = pandas.DataFrame(
+                {f"col{x + 1}": np.arange(nrows) for x in range(ncols)}
+            )
+            getattr(df, func)(filename, **func_kw if func_kw else {})
+            filenames.append(filename)
+
+    file_type_to_extension = {
+        "excel": "xlsx",
+        "fwf": "txt",
+        "pickle": "pkl",
+    }
+    extension = file_type_to_extension.get(file_type, file_type)
+
+    def _make_default_file(filename=None, nrows=NROWS, ncols=2, force=True, **kwargs):
+        if filename is None:
+            filename = get_unique_filename(extension=extension)
+
+        if file_type == "json":
+            lines = kwargs.get("lines")
+            func_kw = {"lines": lines, "orient": "records"} if lines else {}
+            _create_file(filenames, filename, force, nrows, ncols, "to_json", func_kw)
+        elif file_type in ("html", "excel", "feather", "stata", "pickle"):
+            _create_file(filenames, filename, force, nrows, ncols, f"to_{file_type}")
+        elif file_type == "hdf":
+            func_kw = {"key": "df", "format": kwargs.get("format")}
+            _create_file(filenames, filename, force, nrows, ncols, "to_hdf", func_kw)
+        elif file_type == "fwf":
+            if force or not os.path.exists(filename):
+                fwf_data = kwargs.get("fwf_data")
+                if fwf_data is None:
+                    with open("modin/pandas/test/data/test_data.fwf", "r") as fwf_file:
+                        fwf_data = fwf_file.read()
+                with open(filename, "w") as f:
+                    f.write(fwf_data)
+                filenames.append(filename)
+        else:
+            raise ValueError(f"Unsupported file type: {file_type}")
+        return filename
+
+    return _make_default_file, filenames
+
+
+def dict_equals(dict1, dict2):
+    """Check whether two dictionaries are equal and raise an ``AssertionError`` if they aren't."""
+    for key1, key2 in itertools.zip_longest(sorted(dict1), sorted(dict2)):
+        assert (key1 == key2) or (np.isnan(key1) and np.isnan(key2))
+        value1, value2 = dict1[key1], dict2[key2]
+        if is_list_like(value1):
+            np.testing.assert_array_equal(value1, value2)
+        else:
+            assert value1 == value2
